# Copyright (C) 2010 Alexandre Garino
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require_dependency 'issue'

module RedmineScrumROI
  module Compute
    def self.included(base)
      base.send(:include, InstanceMethods)
      base.class_eval do
        unloadable
        before_save :update_roi
      end
    end

    module InstanceMethods

      def get_custom_field(name)
	custom_field_id = Setting.plugin_redmine_scrum_roi["#{name}"]
        if custom_field_id.nil? || custom_field_id.blank?
          return nil
        else
          return self.custom_field_values.detect {|v| v.custom_field_id == custom_field_id.to_i}
        end
      end

      def update_roi

	# Get the ROI Custom Field
	roi = get_custom_field('roi_custom_field')
	if roi.nil?
	  return true
	end

	# Get the Business Value Custom Field
	business_value = get_custom_field('business_value_custom_field')
        business_value = business_value.value if business_value
    	if business_value.blank?
	  roi.value = ""
	  return true
	end

	# Get the Story Point Custom Field
	story_point = get_custom_field('story_point_custom_field')
        story_point = story_point.value if story_point
    	if story_point.blank?
	  roi.value = ""
	  return true
	end

	# Compute the ROI
	if (story_point.to_f == 0.0)
	  new_roi = 0.0
        else 
	  new_roi = business_value.to_f / story_point.to_f
        end

	# Format the ROI
	new_roi = "%.2f" % new_roi

	# Update the ROI Custom Field if necessary
	if (roi.value != new_roi)
	  roi.value = new_roi
        end

        return true
      end
    end
  end
end
